using System;
using Autofac.Extensions.DependencyInjection;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Configuration;
using Serilog;
using MediatR;
using IRaCIS.Core.Application.MediatR.Handlers;
using System.Threading.Tasks;
using MassTransit;
using MassTransit.NewIdProviders;
using System.IO;
using Newtonsoft.Json.Linq;
using System.Security.Cryptography;
using IRaCIS.Core.Infrastructure;
using System.Net.NetworkInformation;
using System.Linq;
using System.Text.Json.Nodes;
using Microsoft.EntityFrameworkCore;
using IRaCIS.Core.Infra.EFCore;
using Microsoft.Data.SqlClient;

namespace IRaCIS.Core.API
{
    public class Program
    {
        public readonly string environment;
        public  static async Task Main(string[] args)
        {



            try
            {
                //ļΪ׼ urlȡֵ(дݲļ˾ͲҪݻ)
                var config = new ConfigurationBuilder()
                            .AddEnvironmentVariables()
                            .Build();

                var enviromentName = config["ASPNETCORE_ENVIRONMENT"];

                if (string.IsNullOrWhiteSpace(enviromentName))
                {

                    var index = Array.IndexOf(args, "--env");
                    enviromentName = index > -1
                        ? args[index + 1]
                        : "Development";
                }

                //Dicom 
                //ImageManager.SetImplementation(WinFormsImageManager.Instance);

                var host = CreateHostBuilder(args)
                    .UseEnvironment(enviromentName) //д뻷
                    .ConfigureAppConfiguration((hostContext, config) =>
                    {

                        //Console.WriteLine(hostContext.HostingEnvironment.EnvironmentName);                
                        config.AddJsonFile("appsettings.json", false, true)
                                  .AddJsonFile($"appsettings.{enviromentName}.json", false, true);
                    })
                    .Build();



                NewId.SetProcessIdProvider(new CurrentProcessIdProvider());




                //// Serilog 
                SerilogExtension.AddSerilogSetup(enviromentName, host.Services);



                #region ֤
                //if (!File.Exists($@"C:\ProgramData\.xingcang\config.json"))
                //{
                //    Console.WriteLine("ǰδע");
                //    Log.Logger.Error("ǰδע");
                //    Console.ReadLine();
                //    return;
                //}
                //else
                //{
                //    var json = File.ReadAllText($@"C:\ProgramData\.xingcang\config.json");

                //    JObject jsonObject = JObject.Parse(json);

                //    var key = jsonObject["key"].ToString();

                //    var value = jsonObject["value"].ToString();


                //    var physicalAddressList = NetworkInterface.GetAllNetworkInterfaces().Select(t => t.GetPhysicalAddress().ToString());

                //    // жļĻǷǱ
                //    if (!physicalAddressList.Contains(key))
                //    {
                //        Console.WriteLine("ͱӦ");
                //        Log.Logger.Error("ͱӦ");
                //        Console.ReadLine();

                //        return;
                //    }


                //    var secrete = MD5Helper.Md5($"{key}_XINGCANG");

                //    if (value != secrete)
                //    {
                //        Console.WriteLine("ע벻ƥ");
                //        Log.Logger.Error("ע벻ƥ");
                //        Console.ReadLine();

                //        return;

                //    }

                //}

                #endregion



                //Ŀ״̬  
                await InitCache(host);



                host.Run();

                Log.Logger.Warning($"ǰ{enviromentName}");

            }
            catch (Exception e)
            {

                Log.Logger.Error(e.InnerException is null ? e.Message + e.StackTrace : e.InnerException?.Message + e.InnerException?.StackTrace);
            }




        }

        public static IHostBuilder CreateHostBuilder(string[] args) =>
            Host.CreateDefaultBuilder(args)
                .UseWindowsService()
                .ConfigureWebHostDefaults(webBuilder =>
                {
                    webBuilder.ConfigureKestrel((context, options) =>
                     {
                         //Ӧ÷KestrelΪ1GB // if don't set default value is: 30 MB
                         options.Limits.MaxRequestBodySize = long.MaxValue;
                         options.Limits.KeepAliveTimeout = TimeSpan.FromMinutes(30);
                         options.Limits.RequestHeadersTimeout = TimeSpan.FromMinutes(20);

                     });
                    //webBuilder.UseSerilog();//ʱserilog,΢ILogger
                    webBuilder.UseStartup<Startup>();
                }).UseSerilog()
                .UseServiceProviderFactory(new AutofacServiceProviderFactory());//ʹAutofac
                                                                                //ConfigureLogging(logging =>
                                                                                //{
                                                                                //    logging.ClearProviders();
                                                                                //    logging.SetMinimumLevel(Microsoft.Extensions.Logging.LogLevel.Trace);
                                                                                //})
                                                                                //.UseNLog()
                                                                                // NLog: Setup NLog for Dependency injection;




        private static async Task InitCache(IHost host)
        {
            var _mediator = host.Services.GetService(typeof(IMediator)) as IMediator;

            await _mediator.Send(new AnonymizeCacheRequest());

            await _mediator.Send(new TrialStateCacheRequest());
        }

    }
}
